# ncert_learn

**ncert_learn** is a comprehensive Python module designed to support NCERT Class 12 Computer Science students. It offers a wide range of utility functions across topics like Python programming, MySQL database interactions, mathematical operations, data structures, system utilities, and much more.

## Key Features

### **Mathematical Functions**
- Prime number check, Fibonacci series, even/odd checks
- Advanced mathematical computations: GCD, LCM, prime factorization, modular exponentiation
- Newly added functions in version 4.9.0:
  - `trigonometric_sine`, `trigonometric_cosine`, `trigonometric_tangent`
  - `quadratic_roots`, `power`, `logarithm`, `factorial`
  - `derivative`, `definite_integral`, `series_sum`
  - Geometric computations like area/volume calculations

### **Data Structures**
- Stack operations: push, pop, peek, and display
- Sorting algorithms: Bubble Sort, Insertion Sort

### **MySQL Operations**
- Database and table management
- Optimized features like `mysql_execute_advanced_mode`

### **File Handling**
- Text, binary, and CSV file management
- ZIP file operations: compress, extract, list contents

### **System Utilities**
- Fetch system and environment details
- Manage services like XAMPP MySQL/Apache

### **Advanced Numerical Functions**
- `numerical_add`, `numerical_subtract`, `numerical_mean`, `numerical_std`
- Matrix operations: inversion, determinant, singular value decomposition

### **Cryptographic Functions**
- Encode/decode methods: Base64, Hex, Caesar cipher, and more

### **Geometric Calculations**
- Area and volume calculations for 2D and 3D shapes:
  - Circles, triangles, spheres, cylinders, cones

### **API Functions**
- Perform CRUD operations:
  - `api_create_item`, `api_read_item`, `api_update_item`, `api_delete_item`
- User management and file handling:
  - `api_create_user`, `api_upload_file`

### **Machine Learning Functions**
- Dataset preprocessing:
  - `handle_missing_values`, `normalize_data`, `standardize_data`
- Models:
  - Linear regression, logistic regression, decision trees, random forests
- Evaluation metrics:
  - `mean_squared_error`, `accuracy_score`
- Visualization:
  - Feature importance plots, decision boundaries

### **Code Quality Tools**
- Format and lint Python code:
  - `format_code`, `lint_code`, `check_code_quality`

### **Search Algorithms**
- Binary, linear, jump, and interpolation search methods

---



## Changelog  

All notable changes to this project are documented in the [Changelog](CHANGELOG.md).  

## Recommendation: Upgrade to Version 4.9.0

We recommend downloading **version 4.9.0**, as it includes important bug fixes and new features that enhance performance, usability, and stability. Upgrade today for an improved experience.

## Disclaimer
This module is intended for educational purposes only. Using this module for any illegal activities is strictly prohibited. The authors and contributors are not responsible for any misuse of the module.

## Installation

To install **ncert_learn**, use `pip`:

```bash
pip install ncert_learn
